@@ -1,4 +1,4 @@
-use std::{cell::RefCell, rc::Rc};
+use std::{cell::RefCell, ops::Range, rc::Rc};
 
 #[derive(Debug)]
 pub enum AstNode {
@@ -48,11 +48,12 @@ pub enum AstNode {
 #[derive(Debug)]
 pub struct Ast {
     pub node: AstNode,
+    pub range: Range<usize>,
     pub label: Option<Vec<String>>,
 }
 
 impl Ast {
-    pub fn new(node: AstNode, label: Option<Vec<String>>) -> Ast {
-        Ast { node, label }
+    pub fn new(node: AstNode, range: Range<usize>, label: Option<Vec<String>>) -> Ast {
+        Ast { node, range, label }
     }
 }
